import { useEffect, useCallback, useState } from 'react';
import useAsyncReducer from 'use-async-reducer';
function getUpdaterPromise(asyncUpdater) {
    if (typeof asyncUpdater === 'function') {
        return asyncUpdater();
    }
    return asyncUpdater;
}
function useRefreshSymbol() {
    const [refreshSymbol, setRefreshSymbol] = useState(() => Symbol());
    const updateSymbol = useCallback(() => {
        setRefreshSymbol(Symbol());
    }, [setRefreshSymbol]);
    return [refreshSymbol, updateSymbol];
}
/**
 * Provides an abstraction to manage async state
 * @param asyncCreator Async method to call, create this method with `useCallback` if it uses state from the component
 * @param options
 * @returns An array of two values
 *  0. State of the async operation, an object with keys `data`, `loading`, `error`
 *  1. An object of dispatch method to update the state
 *
 * @see [https://github.com/azmenak/use-async-call/blob/master/README.md](https://github.com/azmenak/use-async-call/blob/master/README.md)
 */
export default function useAsyncCall(asyncCreator, options = {}) {
    const [refreshSymbol, refresh] = useRefreshSymbol();
    const [response, actions] = useAsyncReducer(options.initialValue);
    useEffect(() => {
        let didCancel = false;
        const callAsync = async () => {
            actions.request();
            try {
                const data = await asyncCreator();
                if (didCancel) {
                    return;
                }
                actions.success(data);
                if (typeof options.onSuccess === 'function') {
                    options.onSuccess(data);
                }
            }
            catch (error) {
                if (didCancel) {
                    return;
                }
                actions.failure(error);
                if (typeof options.onFailure === 'function') {
                    options.onFailure(error);
                }
            }
            finally {
                if (didCancel) {
                    return;
                }
                if (typeof options.onComplete === 'function') {
                    options.onComplete();
                }
            }
        };
        callAsync();
        return () => {
            didCancel = true;
        };
    }, [asyncCreator, refreshSymbol]);
    const update = useCallback(async (asyncUpdater, updateOptions = {}) => {
        actions.request();
        try {
            const data = await getUpdaterPromise(asyncUpdater);
            actions.success(data);
            if (typeof updateOptions.onSuccess === 'function') {
                updateOptions.onSuccess(data);
            }
            return data;
        }
        catch (error) {
            if (updateOptions.saveError) {
                actions.failure(error);
            }
            else {
                actions.complete();
            }
            if (typeof updateOptions.onFailure === 'function') {
                updateOptions.onFailure(error);
            }
            if (updateOptions.throwError) {
                throw error;
            }
        }
        finally {
            if (typeof updateOptions.onComplete === 'function') {
                updateOptions.onComplete();
            }
        }
    }, [actions]);
    return [response, { update, refresh, actions }];
}
