import { Loadable, AsyncReducerBoundActions } from 'use-async-reducer';
export { Loadable } from 'use-async-reducer';
export interface UseAsyncCallOptions<T> {
    /**
     * Initial value used for `data` of state
     */
    initialValue?: T;
    /**
     * Callback called after call is successful
     * @param data Data returned from async caller
     */
    onSuccess?(data?: T): void;
    /**
     * Callback called after async call throws
     * @param error Error thrown by async caller
     */
    onFailure?(error?: Error): void;
    /**
     * Callback always called after async call completes
     */
    onComplete?(): void;
}
export interface UseAsyncCallUpdateOptions<T> {
    /**
     * Should thrown errors be re-thrown in the resulting promise from `update`;
     * useful when using in conjuction with form libraries that expect errors when
     * submitting form values
     */
    throwError?: boolean;
    /**
     * If the caller throws, sets `state.error` to the error and `state.data` to
     * `null`
     */
    saveError?: boolean;
    /**
     * Callback called after call is successful
     * @param data Data returned from async caller
     */
    onSuccess?(data?: T): void;
    /**
     * Callback called after async call throws
     * @param error Error thrown by async caller
     */
    onFailure?(error?: Error): void;
    /**
     * Callback always called after async call completes
     */
    onComplete?(): void;
}
export declare type UseAsyncCallUpdater<T> = (asyncUpdater: Promise<T> | (() => Promise<T>), updateOptions?: UseAsyncCallUpdateOptions<T>) => Promise<T | undefined>;
export declare type UseAsyncCellReturnType<T> = [Loadable<T>, UseAsyncCallReturnTypeOptions<T>];
export interface UseAsyncCallReturnTypeOptions<T> {
    /**
     * Method used to update the state though async calls, accepts a promise or
     * a method which returns a promise, the state data will be updated to the
     * result of the promise
     */
    update: UseAsyncCallUpdater<T>;
    /**
     * When called, the method passed to `useAsyncCall` will be invoked
     */
    refresh(): void;
    /**
     * Provides access to the reducer actions
     */
    actions: AsyncReducerBoundActions<T>;
}
/**
 * Provides an abstraction to manage async state
 * @param asyncCreator Async method to call, create this method with `useCallback` if it uses state from the component
 * @param options
 * @returns An array of two values
 *  0. State of the async operation, an object with keys `data`, `loading`, `error`
 *  1. An object of dispatch method to update the state
 *
 * @see [https://github.com/azmenak/use-async-call/blob/master/README.md](https://github.com/azmenak/use-async-call/blob/master/README.md)
 */
export default function useAsyncCall<T extends any>(asyncCreator: () => Promise<T>, options?: UseAsyncCallOptions<T>): UseAsyncCellReturnType<T>;
